/*
 * CKEditor - The text editor for Internet - http://ckeditor.com
 * Copyright (C) 2003-2008 Frederico Caldeira Knabben
 *
 * == BEGIN LICENSE ==
 *
 * Licensed under the terms of any of the following licenses at your
 * choice:
 *
 *  - GNU General Public License Version 2 or later (the "GPL")
 *    http://www.gnu.org/licenses/gpl.html
 *
 *  - GNU Lesser General Public License Version 2.1 or later (the "LGPL")
 *    http://www.gnu.org/licenses/lgpl.html
 *
 *  - Mozilla Public License Version 1.1 or later (the "MPL")
 *    http://www.mozilla.org/MPL/MPL-1.1.html
 *
 * == END LICENSE ==
 */

CKEDITOR.plugins.add( 'dialog',
	{
		init : function( editor, pluginPath )
		{
		}
	});

CKEDITOR.DIALOG_RESIZE_NONE = 0;
CKEDITOR.DIALOG_RESIZE_RESIZE_WIDTH = 1;
CKEDITOR.DIALOG_RESIZE_RESIZE_HEIGHT = 2;
CKEDITOR.DIALOG_RESIZE_RESIZE_BOTH = 3;

CKEDITOR.dialog =
{
	add: function( name, callback )
	{
	},

	okButton : null,

	cancelButton : null,

	addUIElement: null,

	dialogDefinitions : {
		testOnly : function()
		{
			return {
				title : 'Test Dialog',
				resizable : CKEDITOR.DIALOG_RESIZE_NONE,
				minWidth : 500,
				minHeight : 400,
				contents : [
					{
						id : 'tab1',
						label : 'First Tab',
						title : 'First Tab Title',
						accessKey : 'Q',
						elements : []
					},

					{
						id : 'tab2',
						label : 'Second Tab',
						title : 'Second Tab Title',
						accessKey : 'W',
						elements : []
					},

					{
						id : 'tab3',
						label : 'Third Tab',
						title : 'Third Tab Title',
						accessKey : 'E',
						elements : []
					}
				]
			};
		}
	},

	storedDialogs : {},

	construct : function( editor, dialogName )
	{
		if ( arguments.length < 2 )
			return;

		// Load the dialog definition.
		var definition = CKEDITOR.dialog.dialogDefinitions[dialogName];
		if ( typeof( definition ) != 'function' )
		{
			alert( 'Error: The dialog "' + dialogName + '" is not defined.' );
			return;
		}
		definition = CKEDITOR.tools.extend( {}, CKEDITOR.dialog._.defaultDialogDefinition, definition() );
		if ( !( definition.title && definition.contents ) )
		{
			alert( 'Error: The dialog "' + dialogName + '" is missing its title or contents.' );
			return;
		}
		if ( definition.contents.length < 1 )
		{
			alert( 'Error: The dialog "' + dialogName + '" has no contents.' );
			return;
		}

		// Initialize some basic parameters.
		this._ = {
			editor : editor,
			element : editor.theme.buildDialog( editor ),
			name : dialogName,
			definition : definition,
			isShown : false,
			size : { width : 0, height : 0 }
		};

		// Initialize the parts map.
		var element = this._.element.getFirst();
		this._.parts = {
			'tl' : [0,0],
			't' : [0,1],
			'tr' : [0,2],
			'l' : [1,0],
			'c' : [1,1],
			'r' : [1,2],
			'bl' : [2,0],
			'b' : [2,1],
			'br' : [2,2],
			'title' : [1,1,0],
			'tabs' : [1,1,1],
			'contents' : [1,1,2],
			'buttons' : [1,1,3]
		};
		for ( var i in this._.parts )
			this._.parts[i] = element.getChild( this._.parts[i] );

		// Initialize the tab and page map.
		this._.tabs = {};

		// Insert the title.
		( new CKEDITOR.dom.text( definition.title, CKEDITOR.document ) ).appendTo( this._.parts.title );

		// Insert the tabs and contents.
		for ( var i = 0 ; i < definition.contents.length ; i++ )
			this.addPage( definition.contents[i] );
		this.selectPage( definition.contents[0].id );

		var classRegex = /cke_dialog_tab(\s|$|_)/;
		this._.parts['tabs'].on( 'click', function( evt )
				{
					var target = evt.data.getTarget(), firstNode = target, id, page;

					// If we aren't inside a tab, bail out.
					if ( !classRegex.test( target.$.className ) )
						return;

					// Find the outer <span> container of the tab.
					while ( target.getName() == 'div' && classRegex.test( target.$.className ) )
						target = target.getParent();
					id = target.$.id.substr( 0, target.$.id.lastIndexOf( '_' ) );
					this.selectPage( id );
				}, this );
	}
};

CKEDITOR.dialog._ = {
	defaultDialogDefinition :
	{
		resizable : CKEDITOR.DIALOG_RESIZE_NONE,
		minWidth : 600,
		minHeight : 400,
		buttons : [CKEDITOR.dialog.okButton, CKEDITOR.dialog.cancelButton],
		onOk : function(){alert('ok called');},
		onCancel : function(){alert('cancel caled');},
		onLoad : function(){alert('onload called');}
	}
}


CKEDITOR.dialog.construct.prototype = {
	resize : (function()
	{
		// Define the function for resizing dialog parts at load to speed up
		// the actual resize operation.
		var setSize = function( dialog, partName, width, height )
		{
			var element = partName ? dialog._.parts[partName] : dialog._.element.getFirst();
			element.setStyles( {
				width : Math.max( width || 0, 0 ) + 'px',
				height : Math.max( height || 0, 0 ) + 'px'
			} );
		}

		// Dialog parts dimensions.
		//  16x16  |  ?x16  |  16x16
		//  16x?   |  ?x?   |  16x?
		//  30x51  |  ?x51  |  30x51
		return function( width, height )
		{
			setSize( this, 't', width - 32, 16 );
			setSize( this, 'l', 16, height - 67 );
			setSize( this, 'c', width - 32, height - 67 );
			setSize( this, 'r', 16, height - 67 );
			setSize( this, 'b', width - 60, 51 );
			setSize( this, null, width, height );
			this._.size = { width : width, height : height };
		}
	})(),

	move : function( x, y )
	{
		// The dialog may be fixed positioned or absolute positioned. Ask the
		// browser what is the current situation first.
		var isFixed = this._.element.getFirst().getComputedStyle( 'position' ) == 'fixed';

		// If not fixed positioned, add scroll position to the coordinates.
		if ( !isFixed )
		{
			var scrollPosition = CKEDITOR.document.getWindow().getScrollPosition();
			x += scrollPosition.x;
			y += scrollPosition.y;
		}

		this._.element.getFirst().setStyles(
				{
					'left' : x + 'px',
					'top' : y + 'px'
				});
	},

	show : function()
	{
		// Insert the dialog's element to the root document.
		var element = this._.element;
		var definition = this._.definition;
		if ( element.getParent() != CKEDITOR.document.getBody() )
			element.appendTo( CKEDITOR.document.getBody() );

		// First, set the dialog to an appropriate size.
		this.resize( definition.minWidth, definition.minHeight );

		// Rearrange the dialog to the middle of the window.
		var viewSize = CKEDITOR.document.getWindow().getViewPaneSize();
		this.move( ( viewSize.width - this._.size.width ) / 2, ( viewSize.height - this._.size.height ) / 2 );
	},

	hide : function()
	{
		// Remove the dialog's element from the root document.
		var element = this._.element;
		if ( !element.getParent() )
			return;
		element.remove();
	},

	addPage : function( contents, index )
	{
		var page = CKEDITOR.dom.element.createFromHtml( '<div class="cke_dialog_page_contents"></div>' );
		var tab = CKEDITOR.dom.element.createFromHtml( [
					'<span><div class="cke_dialog_tab">',
						'<div class="cke_dialog_tab_left"></div>',
						'<div class="cke_dialog_tab_center">',
							CKEDITOR.tools.htmlEncode( contents.label ),
						'</div>',
						'<div class="cke_dialog_tab_right"></div>',
					'</div></span>'
				].join( '' ) );
		this._.tabs[ contents.id ] = [ tab, page ];
		tab.getFirst().unselectable();
		page.appendTo( this._.parts['contents'] );
		tab.appendTo( this._.parts['tabs'] );
		tab.setAttribute( 'id', contents.id + '_' + CKEDITOR.tools.getNextNumber() );
	},

	selectPage : function( id )
	{
		for ( var i in this._.tabs )
		{
			var tab = this._.tabs[i][0],
				page = this._.tabs[i][1];
			if ( i == id )
			{
				tab.addClass( 'cke_dialog_tab_selected');
				page.show();
			}
			else
			{
				tab.removeClass( 'cke_dialog_tab_selected' );
				page.hide();
			}
		}
	}
};

(function()
{
	CKEDITOR.tools.extend( CKEDITOR.editor.prototype,
		{
			'openDialog' : function( dialogName )
			{
				var dialog = CKEDITOR.dialog.storedDialogs[dialogName] || new CKEDITOR.dialog.construct( this, dialogName );
				CKEDITOR.dialog.storedDialogs[dialogName] = dialog;
				dialog.show();
				return dialog;
			}
		});
})();
